slib = {
	lang = { }
}

setmetatable(slib, {
	__index = function( self, key )
		return slib : get(key)
	end,
})

-- ============================================================================
-- collection to inherit

slib.collection = {
	__call = function(self, ...)
		if table.getn(self) == 0 then return end
		if self.rnd == true then
			local last = self.it
			if not last or table.getn(self) > 1 then
				self.it = last
				while self.it == last do
					self.it = math.random(table.getn(self))
				end
			else
				self.it = math.random(table.getn(self))
			end
		else
			self.it = self.it or 1
		end
		local text
		if table.getn(arg) > 0 then
			text = string.format(self[self.it], unpack(arg))
		else
			text = self[self.it]
		end
		if not self.rnd then
			self.it = math.mod(self.it, table.getn(self)) + 1
		end
		--print "\n"
		return text
	end,
}

-- ============================================================================
-- add new language library

function slib : add( language, library )
	assert(type(language)=="string" and type(library)=="string", "wrong arguments")
	library = string.gsub(library, "\\\n", "\r")
	
	local lib, it = { }
	string.gsub(library .. "\n", "(.-)\n", function( s )
		local green = false
		string.gsub(s, "%[([#]*)([%w.]+)%]", function( random, addr )
			it, green = lib, true
			string.gsub(addr, "%w+", function( name )
				if it[name] then
					it = it[name]
				else
					local coll = { rnd = (string.len(random) ~= 0) }
					setmetatable(coll, self.collection)
					it, it[name] = coll, coll
				end
			end)
		end)
		
		if not green and it then
			string.gsub(s, "%s*(%S.-)%s*$", function( phrase )
				phrase = string.gsub(phrase, "\r", "\n")
				table.insert(it, phrase)
			end)
		end
	end)
	
	self.lang[language] = lib
end

-- ============================================================================
-- set current language

function slib : set( language )
	assert(self.lang[language], "language " .. language .. " is missing in the library")
	self.current = language
end

-- ============================================================================
-- get top-level phrase by id (current language used)

function slib : get( id )
	assert(self.current, "the current language isn't set")
	local phrase = self.lang[self.current][id]
	assert(phrase, string.format("unknown phrase %q in %q ", id, self.current))
	return phrase
end

function slib : has( id )
	local phrase = self.lang[self.current][id]
	return phrase ~= nil
end

-- ============================================================================

dofile "slib.lua"
